package pipeClient;

import java.io.IOException;
import java.io.RandomAccessFile;
import java.io.UnsupportedEncodingException;
import java.util.HashMap;
import java.util.LinkedList;

import pipeClient.PacketHeader.DataBlockTypeEnum;

public class ByteStream 
{
	private final String encoding = "UTF-16LE";
	private RandomAccessFile ioStream;
	private int maxPacketSize;
	
	public int getMaxPacketSize() 
	{
		return maxPacketSize;
	}
	public void setMaxPacketSize(int maxPacketSize) 
	{
		this.maxPacketSize = maxPacketSize;
	}
	
	public ByteStream(RandomAccessFile ioStream) 
	{
		this.ioStream = ioStream;
	}
	
	public void writeHandshake(String id) throws IOException 
	{
		try 
		{
			byte[] msg = id.getBytes(encoding);
			
			PacketHeader hdr = new PacketHeader();
			hdr.setDataBlockType(DataBlockTypeEnum.Handshake);
			hdr.setDataBlockSize(msg.length);
			hdr.setItemSize(msg.length);
			byte[] hdrPacket = hdr.toArray();
			
			int packetSize = hdrPacket.length + msg.length;
			byte[] packet = new byte[packetSize];
			System.arraycopy(hdrPacket, 0, packet, 0, hdrPacket.length);
			System.arraycopy(msg, 0, packet, hdrPacket.length, msg.length);
				
			ioStream.write(packet);
			
		} 
		catch (UnsupportedEncodingException e) 
		{
			e.printStackTrace();
		}
	}
	
	public String readHandshake() throws IOException 
	{
		PacketHeader hdr = readPacketHeader();
		byte[] dataBlock = readPacketDataBlock(hdr.getDataBlockSize());
		
		LinkedList<String> items = extractStrings(dataBlock, hdr.getItemSize());
		return items.getFirst();
	}
	
	public void writeMonitorInfo(int value) throws IOException
	{
		try 
		{
			byte[] valuePacket = BitConverter.toBytes(value);
			
			PacketHeader hdr = new PacketHeader();
			hdr.setDataBlockType(DataBlockTypeEnum.MonitorInfo);
			hdr.setDataBlockSize(valuePacket.length);
			hdr.setItemSize(valuePacket.length);
			byte[] hdrPacket = hdr.toArray();
			
			int packetSize = hdrPacket.length + valuePacket.length;
			byte[] packet = new byte[packetSize];
			System.arraycopy(hdrPacket, 0, packet, 0, hdrPacket.length);
			System.arraycopy(valuePacket, 0, packet, hdrPacket.length, valuePacket.length);
				
			ioStream.write(packet);
			
		} 
		catch (UnsupportedEncodingException e) 
		{
			e.printStackTrace();
		}
	}
	
	public int readMonitorInfo() throws IOException 
	{
		PacketHeader hdr = readPacketHeader();
		byte[] dataBlock = readPacketDataBlock(hdr.getDataBlockSize());
		
		LinkedList<Integer> items = extractIntegers(dataBlock, hdr.getItemSize());
		return items.getFirst();
	}
	
	public PacketHeader readPacketHeader() throws IOException 
	{
		PacketHeader hdr = new PacketHeader();
		int hdrSize = hdr.getLength();
		byte[] array = new byte[hdrSize];
		int bytesRead = ioStream.read(array, 0, hdrSize);
		if (bytesRead == hdrSize) 
		{
			hdr.set(array);
		}
		
		return hdr;
	}	
	
	public byte[] readPacketDataBlock(int size) throws IOException 
	{
		byte[] dataBlock = new byte[size];
		ioStream.read(dataBlock, 0, size);
		return dataBlock;
	}
	
	public LinkedList<String> extractStrings(byte[] dataBlock, int itemSize) 
	{
		LinkedList<String> values = new LinkedList<String>();
		int index = 0;
		while (index < dataBlock.length) 
		{
			byte[] valueArray = new byte[itemSize];
			System.arraycopy(dataBlock, index, valueArray, 0, itemSize);
			try
			{
				values.add(new String(valueArray, encoding));
			} 
			catch (UnsupportedEncodingException e)
			{
				e.printStackTrace();
			}
			index += itemSize;
		}
		return values;
	}
	
	public LinkedList<Integer> extractIntegers(byte[] dataBlock, int itemSize)
	{
		LinkedList<Integer> values = new LinkedList<Integer>();
		int index = 0;
		while (index < dataBlock.length) 
		{
			byte[] valueArray = new byte[itemSize];
			System.arraycopy(dataBlock, index, valueArray, 0, itemSize);
			values.add(BitConverter.toInt(dataBlock,  0));
			index += itemSize;
		}
		return values;
	}
	
	public HashMap<Integer, LinkedList<Spike>> extractSpikes(byte[] dataBlock, int itemSize)
	{
		HashMap<Integer, LinkedList<Spike>> spikes = new HashMap<Integer, LinkedList<Spike>>();
		int index = 0;
		while (index < dataBlock.length) 
		{
			byte[] spikeArray = new byte[itemSize];
			System.arraycopy(dataBlock, index, spikeArray, 0, itemSize);
			Spike spike = new Spike(spikeArray);
			int sensorID = spike.getSensorID();
			if (!spikes.containsKey(sensorID))
			{
				spikes.put(sensorID, new LinkedList<Spike>());
			}
			spikes.get(sensorID).add(spike);
			index += itemSize;
		}
		return spikes;
	}
	
	public Event extractEvent(byte[] dataBlock)
	{
		Event evt = new Event(dataBlock);
		
		return evt;
	}
}
