package pipeClient;

import java.io.FileNotFoundException;
import java.io.IOException;
import java.io.RandomAccessFile;
import java.util.HashMap;
import java.util.LinkedList;

import pipeClient.PacketHeader.DataBlockTypeEnum;

public class PipeClient {
	
	public enum StateEnum
	{
		Connected,
		Disconnected
	}
	
	public class StateEvent
	{
		private StateEnum state;
		private String message;
		
		public StateEnum getState()
		{
			return state;
		}
		public void setState(StateEnum state)
		{
			this.state = state;
		}
		public String getMessage()
		{
			return message;
		}
		public void setMessage(String message)
		{
			this.message = message;
		}
	}
	
	public enum EventEnum
	{
		ReceivedData,
		ConnectionTimeOut,
		TaskCanceled,
		BrokenPipe,
		TransferFailed,
		UnhandledException
	}
	
	public class EventEvent 
	{
		private EventEnum event;
		private String message;
		
		public EventEnum getEvent()
		{
			return event;
		}
		public void setEvent(EventEnum event)
		{
			this.event = event;
		}
		public String getMessage()
		{
			return message;
		}
		public void setMessage(String message)
		{
			this.message = message;
		}
	}
	
	private LinkedList<PipeClientListener> listeners;
	private StateEnum state;
	private RandomAccessFile pipe;
	private ByteStream ioStream;
	
	public StateEnum getState()
	{
		return state;
	}

	public void setState(StateEnum state)
	{
		this.state = state;
	}

	public PipeClient()
	{
		listeners = new LinkedList<PipeClientListener>();
		state = StateEnum.Disconnected;
	}
	
	public void addListener(PipeClientListener listener)
	{
		listeners.add(listener);
	}
	
	public void start(String machineName)
	{
		try
		{
			pipe = new RandomAccessFile(machineName, "rw");
			//pipe = new RandomAccessFile("test", "rw");
			ioStream = new ByteStream(pipe);
		} 
		catch (FileNotFoundException e)
		{
			e.printStackTrace();
			sendEvent(EventEnum.UnhandledException, e.getMessage());
		}
	}
	
	public boolean connect()
	{
		try
		{
			ioStream.writeHandshake(Security.ClientSignature);
			
			String message = ioStream.readHandshake();
			if (message.equals(Security.ServerSignature))
			{
				sendState(StateEnum.Connected, "Server/Client handshake successful. Connected to " + message + ".");
			}
			else 
			{
				return false;
			}
		} 
		catch (IOException e)
		{
			e.printStackTrace();
			sendEvent(EventEnum.BrokenPipe, e.getMessage());
			return false;
		}
		
		return true;
	}
	
	public void close()
	{
		try
		{
			pipe.close();
			sendState(StateEnum.Disconnected, "Disconnected from server.");
		} 
		catch (IOException e)
		{
			e.printStackTrace();
			sendEvent(EventEnum.UnhandledException, e.getMessage());
		}
	}
	
	public void listenForData()
	{
		try
		{ 
			PacketHeader hdr = ioStream.readPacketHeader();
			byte[] dataBlock = ioStream.readPacketDataBlock(hdr.getDataBlockSize());
			
			if (hdr.getDataBlockType() == DataBlockTypeEnum.SpikeData)
			{
				HashMap<Integer, LinkedList<Spike>> spikes = ioStream.extractSpikes(dataBlock, hdr.getItemSize());
				
				for (PipeClientListener l : listeners)
				{
					l.signalSpikeData(spikes);
				}
			}
			else if (hdr.getDataBlockType() == DataBlockTypeEnum.Event)
			{
				Event evt = ioStream.extractEvent(dataBlock);
				
				for (PipeClientListener l : listeners)
				{
					l.signalEventData(evt);
				}
			}
		} 
		catch (IOException e)
		{
			sendEvent(EventEnum.BrokenPipe, e.getMessage());
			e.printStackTrace();
			close();
		}
	}
	
	private void sendEvent(EventEnum event, String message)
	{
		EventEvent evt = new EventEvent();
		evt.setEvent(event);
		evt.setMessage(message);
		
		for (PipeClientListener l : listeners)
		{
			l.signalClientEvent(evt);
		}
	}
	
	private void sendState(StateEnum state, String message)
	{
		StateEvent evt = new StateEvent();
		evt.setState(state);
		evt.setMessage(message);
		
		for (PipeClientListener l : listeners)
		{
			l.signalClientState(evt);
		}
	}
}
