function [dataInRange, residuals] = retrieveData(listener, to, residuals)
% function [dataInRange, residuals] = retrieveData(listener, to, residuals)
%
% Retrieves spike data from the McsCMOSPipeClientThread listener object and
% clears the spike data currently stored in the listener object. Returns
% the spikes up to timestamp "to" as dataInRange and any spikes after that
% in residuals.
%
% Input:
%   listener            -   a pipeClientThread.McsCMOSPipeClientThread object
%   to                  -   scalar timestamp in microseconds. This is the
%                           last timestamp included in the range, all
%                           spikes with a later timestamp are stored in
%                           residuals.
%   residuals           -   (65, 65) cell array of spikes too late for the last
%                           retrieval. These are directly copied to
%                           dataInRange.
%
% Output:
%   dataInRange         -   (65 x 65) cell array. Each cell stores a (1 x m)
%                           vector of the collected spikes for a single
%                           sensor. A spike struct has
%                           fields 'SensorID', 'UnitID', 'HasWaveForm' and
%                           'WaveForm'. Field 'WaveForm' is empty is
%                           'HasWaveForm' is false.
%   residuals           -   (65 x 65) cell array. Each cell stores a (1 x m)
%                           vector of the collected spikes for a single
%                           sensor if there are any spikes for this sensor
%                           with a timestamp later than "to". A spike struct has
%                           fields 'SensorID', 'UnitID', 'HasWaveForm' and
%                           'WaveForm'. Field 'WaveForm' is empty is
%                           'HasWaveForm' is false.

    spikeData = handle(listener.getSpikeData);
    dataInRange = residuals;
    residuals = cell(1,65*65);
    sensorIter = spikeData.keySet.iterator;
    while hasNext(sensorIter)
        sensor = next(sensorIter);
        spikeList = get(spikeData,java.lang.Integer(sensor));
        spikeCount = size(spikeList);
        listIter = iterator(spikeList);
        first = true;
        while hasNext(listIter)
            s = next(listIter);
            spike = struct(s);
            if first
                first = false;
                spikesInside = MakeArray(spike, spikeCount);
                nInside = 0;
                spikesOutside = spikesInside;
                nOutside = 0;
            end
            
            if spike.timeStamp > to
                nOutside = nOutside + 1;
                spikesOutside(nOutside) = spike;
            else
                nInside = nInside + 1;
                spikesInside(nInside) = spike;
            end
        end
        if nInside > 0
            sensorID = spikesInside(1).sensorID;
            dataInRange{sensorID} = [dataInRange{sensorID} spikesInside(1:nInside)];
        end
        if nOutside > 0
            sensorID = spikesOutside(1).sensorID;
            residuals{sensorID} = spikesOutside(1:nOutside);
        end
    end
end

function arr = MakeArray(spike, spikeCount)
    arr(spikeCount) = spike;
    arr(:) = spike;
end